#include <stdio.h>
#include "eztcplib.h"

struct spe_env *env_base;
struct wlan_env *wenv_base;
struct etc_opt *etc_base;
struct stat_env	env_stat;

int eztcp_cnt = 0;

void print_error(int errnum)
{
	printf("ErrNum %d, %s\r\n", errnum, strerror(errnum));
}

void remote_search_eztcp(DWORD ip, struct spe_env* env, struct wlan_env* wenv, struct etc_opt* etc)
{
	int nResult, nCount, nErr, i;
	struct in_addr in;

	nResult = RemoteReadEzTCP(ip, env, wenv, etc, &nCount, &nErr);

	if ( nResult == EZTCP_ERR )
	{
		eztcp_cnt = 0;
		print_error(nErr);
	}
	else
	{
		printf("Found %d ezTCP\r\n", nCount);
		eztcp_cnt = nCount;
		for ( i = 0 ; i < nCount ; i++ )
		{
			in.s_addr = (env_base + i)->local_ip;

			printf("[No %d] %02X:%02X:%02X:%02X:%02X:%02X \tIP=%s\n", 
			i+1,
			(env_base + i)->eth_addr[0],
			(env_base + i)->eth_addr[1],
			(env_base + i)->eth_addr[2],
			(env_base + i)->eth_addr[3],
			(env_base + i)->eth_addr[4],
			(env_base + i)->eth_addr[5],
			inet_ntoa(in)
			);
		}
	}	
}

void password_test(struct spe_env* env, int set_or_change_or_clear, const char* pwd, const char* new_pwd)
{
	int nResult, nErr;
	char ChangePwd[12];

	if ( set_or_change_or_clear == 1 ) /* set password */
	{
		memset(ChangePwd, 0x00, 12);
		memset(env->passwd, 0x00, sizeof(env->passwd));

		memcpy(ChangePwd, pwd, strlen(pwd));
	}
	else if ( set_or_change_or_clear == 2 ) /* change password */
	{
		memset(env->passwd, 0x00, sizeof(env->passwd));
		memcpy(env->passwd, pwd, strlen(pwd));

		memset(ChangePwd, 0x00, 12);
		memcpy(ChangePwd, new_pwd, strlen(new_pwd));
	}
	else if ( set_or_change_or_clear == 3 ) /* delete password */
	{
		memset(env->passwd, 0x00, sizeof(env->passwd));
		memcpy(env->passwd, pwd, strlen(pwd));

		memset(ChangePwd, 0x00, 12);
	}
	
	nResult = ChangePwdEzTCP(env, ChangePwd, &nErr);

	if ( nResult == 1 )
	{
		printf("Password Write Success.!!!");
	}
	else
	{
		if ( nErr == EZTCP_ERR_RES )
		{
			printf("Password Write Error...");
		}
		else if ( nErr == EZTCP_ERR_PWD )
		{
			printf("Password mismatch Error...");
		}
		else
		{
			printf("Unknown Error...");
		}
	}


}

void status_test(struct spe_env* env)
{
	int nResult, nErr;
	struct stat_env env_stat;


	memset(&env_stat, 0, sizeof(struct stat_env));
		
	nResult = StatusEzTCP(env, &env_stat, &nErr);
	
	if ( nResult == 1 )
	{
		printf("########################################\r\n");
		printf("Firmware Version : %s\r\n", env_stat.ver);
		printf("Frequency : %s\r\n", env_stat.freq);

		printf("Uptime : %d days, %02u:%02u:%02u\r\n\r\n",
			env_stat.uptime_day,
			env_stat.uptime_hour,
			env_stat.uptime_minute,
			env_stat.uptime_second
			);

		printf("MAC Address : %02X:%02X:%02X:%02X:%02X:%02X\r\n", 
			env_stat.mac_addr[0],
			env_stat.mac_addr[1],
			env_stat.mac_addr[2],
			env_stat.mac_addr[3],
			env_stat.mac_addr[4],
			env_stat.mac_addr[5]
			);

		printf("IP Address : %d.%d.%d.%d\r\n", 
			env_stat.ip_addr[0],
			env_stat.ip_addr[1],
			env_stat.ip_addr[2],
			env_stat.ip_addr[3]
			);

		printf("Subnet mask : %d.%d.%d.%d\r\n", 
			env_stat.sub_mask[0],
			env_stat.sub_mask[1],
			env_stat.sub_mask[2],
			env_stat.sub_mask[3]
			);

		printf("Gateway : %d.%d.%d.%d\r\n\r\n", 
			env_stat.gate_addr[0],
			env_stat.gate_addr[1],
			env_stat.gate_addr[2],
			env_stat.gate_addr[3]
			);


		printf("Serial RX bytes: %lu\r\n", env_stat.sio_rx);
		
		printf("Serial TX bytes: %lu\r\n", env_stat.sio_tx);
		
		printf("LAN RX packets: %lu\r\n", env_stat.eth_rx);
		
		printf("LAN TX packets: %lu\r\n", env_stat.eth_tx);

		printf("========================================\r\n");
		printf("%s", env_stat.text);
		printf("========================================\r\n");
		printf("########################################\r\n");
	}
	else
	{
		print_error(nErr);
	}
}

void write_test(struct spe_env* env, struct wlan_env* wenv, struct etc_opt* etc, const char* pwd)
{
	int nResult, nErr;

	struct spe_env 	temp_env;
	struct wlan_env temp_wenv; 
	struct etc_opt	temp_etc;

	memcpy(&temp_env, env, sizeof(struct spe_env));
	memcpy(&temp_wenv, wenv, sizeof(struct wlan_env));
	memcpy(&temp_etc, etc, sizeof(struct etc_opt));

	//=====================================================================
	// example.
	
	temp_env.local_ip = MK_IP(10,1,0,123);
	temp_env.local_port = 50000;

	temp_env.sio_baud = 38400;

	memset(temp_etc.comment, 0x00, sizeof(temp_etc.comment));
	sprintf((char*)temp_etc.comment, "%s", "Test Comment");
	
	if ( pwd != NULL )
	{
		memset(temp_env.passwd, 0, 12);
		memcpy(temp_env.passwd, pwd, strlen(pwd));
	}
	else
	{
		memset(temp_env.passwd, 0, 12);
	}
	//=====================================================================
	//
	nResult = WriteEzTCP(&temp_env, wenv, &temp_etc, &nErr);

	if ( nResult == 1 )
	{
		printf("Write Success.!!!\r\n");

		memcpy(env, &temp_env, sizeof(struct spe_env));
		memcpy(etc, &temp_etc, sizeof(struct etc_opt));
	}
	else
	{
		if ( nErr == EZTCP_ERR_RES )
		{
			printf("Write Error...");
		}
		else if ( nErr == EZTCP_ERR_PWD )
		{
			printf("Password mismatch Error...");
		}
		else
		{
			printf("Unknown Error...");
		}
	}
}

void search_eztcp(struct spe_env* env, struct wlan_env* wenv, struct etc_opt* etc)
{
	int nResult, nCount, nErr, i;
	struct in_addr in;

	nResult = ProbeEzTCP(env, wenv, etc, &nCount, &nErr);

	if ( nResult == EZTCP_ERR )
	{
		eztcp_cnt = 0;
		print_error(nErr);
	}
	else
	{
		printf("Found %d ezTCP\r\n", nCount);
		eztcp_cnt = nCount;
		for ( i = 0 ; i < nCount ; i++ )
		{
			in.s_addr = (env_base + i)->local_ip;

			printf("[No %d] %02X:%02X:%02X:%02X:%02X:%02X \tIP=%s\n", 
			i+1,
			(env_base + i)->eth_addr[0],
			(env_base + i)->eth_addr[1],
			(env_base + i)->eth_addr[2],
			(env_base + i)->eth_addr[3],
			(env_base + i)->eth_addr[4],
			(env_base + i)->eth_addr[5],
			inet_ntoa(in)
			);
		}
	}	
}

int main(void)
{
	DWORD eztcp_ip;
	char lib_ver[128];
	int i, nLen, res, err;
	char pwd[8];

	env_base = (struct spe_env*)malloc(sizeof(struct spe_env) * 256);
	wenv_base	= (struct wlan_env *)malloc(sizeof(struct wlan_env) * 256);
	etc_base	= (struct etc_opt *)malloc(sizeof(struct etc_opt) * 256);

	memset(env_base, 0, sizeof(struct spe_env)*256);
	memset(wenv_base, 0, sizeof(struct wlan_env)*256);
	memset(etc_base, 0, sizeof(struct etc_opt)*256);
	
	GetLibVerEzTCP(lib_ver, 128);
	printf("%s\r\n", lib_ver);
	
	eztcp_ip = ntohl(MK_IP(10,1,0,4));
	
	search_eztcp(env_base, wenv_base, etc_base);
	//remote_search_eztcp(eztcp_ip, env_base, wenv_base, etc_base);

	if ( eztcp_cnt > 0 )
	{
		for ( i = 0 ; i < eztcp_cnt ; i++ )
    {
			if ( (env_base+i)->eth_addr[3] == 0x01
      		&& (env_base+i)->eth_addr[4] == 0x63
    			&& (env_base+i)->eth_addr[5] == 0xe6 )
			{
        // read status of ezTCP
        //status_test(env_base+i);

        // write test
        //write_test((env_base+i), (wenv_base+i), (etc_base+i), pwd);

        // set password
        // maximun password length is 8bytes
        //memcpy(pwd, "01234567", 8);
        //password_test((env_base+i), 1, pwd, new_pwd);

        // change password
        // memcpy(pwd, "01234567", 8);
        // memcpy(new_pwd, "abcdefgh", 8);
        // password_test((env_base+i), 2, pwd, new_pwd);

        // delete password
        //memcpy(pwd, "abcdefgh", 8);
      	//password_test((env_base+i), 3, pwd, new_pwd);
				//
				
				// 
				memset(pwd, 0x00, 8);
				memcpy(pwd, "1111", 4);
				/*
				res = ResetEzTCP((env_base+i), pwd, &err);
				if ( res == 1 )
				{
					printf("reset Success.!!!\r\n");
				}
				else
				{
					if ( err == EZTCP_ERR_RES )
					{
						printf("reset Error...");
					}
					else if ( err == EZTCP_ERR_PWD )
					{
						printf("Password mismatch Error...");
					}
					else
					{
						printf("Unknown Error...");
					}
				}*/
				
				res = CloseTCP((env_base+i), pwd, &err);
				if ( res == 1 )
				{
					printf("close Success.!!!\r\n");
				}
				else
				{
					if ( err == EZTCP_ERR_RES )
					{
						printf("close Error...");
					}
					else if ( err == EZTCP_ERR_PWD )
					{
						printf("Password mismatch Error...");
					}
					else
					{
						printf("Unknown Error...");
					}
				}
    	}
  	}
	}


	free (env_base);
	free (wenv_base);
	free (etc_base);
	return 0;
}
